package com.xebia.search;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.FSDirectory;

import parse.ConfigXMLParser;
import parse.LuceneConfiguration;

import com.xebia.query.create.QueryFactory;
import com.xebia.query.create.QueryFactoryImpl;
import com.xebia.query.execute.QueryExecutor;
import com.xebia.query.execute.QueryExecutorImpl;

public class LuceneSearcherImpl implements LuceneSearcher {

    private LuceneConfiguration luceneConfiguration;

    private QueryFactory queryFactory;

    private QueryExecutor queryExecutor;

    private SearcherProvider searcherProvider;

    private Searcher searcher;

    public LuceneSearcherImpl() {
        ConfigXMLParser configXMLParser = new ConfigXMLParser();
        luceneConfiguration = configXMLParser.parse();
        searcherProvider = new SearcherProviderImpl(luceneConfiguration.getIndexLocation());
        searcher = searcherProvider.getSearcher();
        queryFactory = new QueryFactoryImpl(luceneConfiguration);
        queryExecutor = new QueryExecutorImpl();
    }

    private List<String> getValues(String fieldName, Query query) {
        List<String> values = new ArrayList<String>();
        try {
            TopDocs topDocs = queryExecutor.executeQuery(query, searcher);
            ScoreDoc[] scoreDocs = topDocs.scoreDocs;
            for (ScoreDoc scoreDoc : scoreDocs) {
                int docId = scoreDoc.doc;
                Document document = searcher.doc(docId);
                values.add(document.get(fieldName));
            }
        } catch (IOException e) {
            System.out.println("*******Exception occurred while searching**********");
        }
        return values;
    }

    private IndexReader getIndexReader() {
        IndexReader indexReader = null;
        try {
            indexReader = IndexReader.open(FSDirectory.open(luceneConfiguration.getIndexLocation()));
        } catch (CorruptIndexException e) {
            System.out.println("Error opening index reader. Corrupted Index");
        } catch (IOException e) {
            System.out.println("Error opening index. IOException");
        }
        return indexReader;
    }

    public Searcher getSearcher() {
        SearcherProvider searcherProvider = new SearcherProviderImpl(luceneConfiguration.getIndexLocation());
        return searcherProvider.getSearcher();
    }

    public List<String> search(String fieldName, String termToSearch) {
        Query query = queryFactory.createExactQuery(fieldName, termToSearch);
        List<String> values = getValues(fieldName, query);
        return values;
    }

    public List<String> getAll(String fieldName) {
        Query query = queryFactory.createExactQuery(fieldName, "*");
        List<String> values = getValues(fieldName, query);
        return values;
    }

    public String getBestmatch(String fieldName, String termToSearch) {
        Query query = queryFactory.createExactQuery(fieldName, termToSearch);
        List<String> values = getValues(fieldName, query);
        return values.get(0);
    }

    public void deleteFields(String... fieldNames) {
        IndexReader indexReader = getIndexReader();
        try {
            for (String fieldName : fieldNames) {
                indexReader.deleteDocuments(new Term(fieldName));
            }
        } catch (Exception e) {
            System.out.println("Error Deleting documents");
        }
    }

    public void deleteMatchingDocuments(String termValue, String... fieldNames) {
        IndexReader indexReader = getIndexReader();
        try {
            for (String fieldName : fieldNames) {
                indexReader.deleteDocuments(new Term(fieldName));
            }
        } catch (Exception e) {
            System.out.println("Error Deleting documents");
        }
    }

}