package parse;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class ConfigXMLParser {

    private LuceneConfiguration luceneConfiguration;

    public ConfigXMLParser() {
        luceneConfiguration = new LuceneConfiguration();
    }

    public LuceneConfiguration parse() {
        try {
            Element documentElement = getDocumentElement();

            luceneConfiguration.setIndexLocation(new File(getParentNode(documentElement, "indexLocation")));
            luceneConfiguration.setSourceFileLocation(new File(getParentNode(documentElement, "sourceFileLocation")));
            luceneConfiguration.setOverWrite(getParentNode(documentElement, "overWrite"));
            luceneConfiguration.setSeparator(getParentNode(documentElement, "separator"));

            NodeList nodeLst = documentElement.getElementsByTagName("field");
            for (int s = 0; s < nodeLst.getLength(); s++) {
                populateFieldInfo(nodeLst, s);
            }
        } catch (Exception e) {
            System.out.println("************************Error parsing XML******************");
        }
        return luceneConfiguration;
    }

    private Element getDocumentElement() throws ParserConfigurationException, SAXException, IOException {
        File file = new File("src" + File.separator + "main" + File.separator + File.separator + "resources" + File.separator + "lucene.xml");
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
        Document document = documentBuilder.parse(file);
        Element documentElement = document.getDocumentElement();
        documentElement.normalize();
        return documentElement;
    }

    private String getParentNode(Element element, String nodeName) {
        NodeList childNodes = element.getChildNodes();
        for (int i = 0; i < childNodes.getLength(); i++) {
            if (childNodes.item(i).getNodeName().equalsIgnoreCase(nodeName)) {
                return childNodes.item(i).getTextContent();
            }
        }
        return null;
    }

    private void populateFieldInfo(NodeList nodeLst, int s) {
        Node firstNode = nodeLst.item(s);
        if (firstNode.getNodeType() == Node.ELEMENT_NODE) {

            Element fieldElement = (Element) firstNode;

            String fieldName = getNode(fieldElement, "name");
            FieldInfo fieldInfo = new FieldInfo(fieldName);

            String type = getNode(fieldElement, "type");
            fieldInfo.setType(type);

            String analyzer = getNode(fieldElement, "analyzer");
            fieldInfo.setAnalyzer(analyzer);

            String store = getNode(fieldElement, "store");
            fieldInfo.setStore(store);

            luceneConfiguration.addFieldInfo(fieldName, fieldInfo);

            /*
             * System.out.println("Name : " + fieldName);
             * System.out.println("Type : " + type);
             * System.out.println("Analyzer : " + analyzer);
             * System.out.println("Store: " + store);
             */

        }
    }

    private String getNode(Element fstElmnt, String nodeName) {
        NodeList typeElementList = fstElmnt.getElementsByTagName(nodeName);
        Element typeElement = (Element) typeElementList.item(0);
        NodeList typeList = typeElement.getChildNodes();
        String type = ((Node) typeList.item(0)).getNodeValue();
        return type;
    }

    public static void main(String[] args) {
        new ConfigXMLParser().parse();
    }
}